#include "systemservicelauncher.h"
#include "../../ukuiserviceinterface/serviceinterface.h"
#include "../../ukuiserviceinterface/serviceobject.h"

#include <QPluginLoader>

#include <QDBusConnection>
#include <QJsonObject>
#include <QJsonArray>
#include <QJsonValue>

#include <QDBusError>

SystemServiceLauncher::SystemServiceLauncher(QObject *parent)
    : ServiceLauncher{parent}
{

}

bool SystemServiceLauncher::launchServices(const QString &pluginPath, const QStringList &serviceNames, bool resident)
{
    Q_UNUSED(resident)
    // 加载一个插件中指定的服务，todo：处理非常驻服务的生命周期管理
    QPluginLoader l;
    //l.setFileName("/home/kylin/git/ukui-service-manager/service-plugin-demo/libservice-plugin-demo.so");
    l.setFileName(pluginPath);

    bool pluginLoaded = l.load();
    Q_ASSERT(pluginLoaded);
    if (!pluginLoaded)
        return false;

    auto instance = l.instance();
    auto plugin = dynamic_cast<UKUI::ServiceInterface *>(instance);
    Q_ASSERT(plugin);
    if (!plugin)
        return false;

    auto services = plugin->getServiceObjects();
    Q_ASSERT(services.count() > 0);
    if (services.isEmpty())
        return false;

    bool successed = true;

    auto jsonObjValue = l.metaData().value("MetaData");
    auto jsonObj = jsonObjValue.toObject();
    auto servicesArray = jsonObj["services"].toArray();
    for (auto serviceValue: servicesArray) {
        auto serviceObject = serviceValue.toObject();
        QString serviceName = serviceObject["name"].toString();

        // 如果待启动的服务不包含此插件的某一个服务，则跳过注册环节
        if (!serviceNames.contains(serviceName)) {
            continue;
        }

        QString servicePath = serviceObject["path"].toString();
        QString serviceInterface = serviceObject["interface"].toString();
        qDebug() << jsonObj;

        Q_ASSERT(!serviceName.isEmpty());
        Q_ASSERT(!servicePath.isEmpty());
        Q_ASSERT(!serviceInterface.isEmpty());

        if (services.isEmpty()) {
            qCritical() << "配置信息和服务不匹配";
            break;
        }
        auto service = services.takeFirst();

        bool serviceRigistered = QDBusConnection::systemBus().registerService(serviceName);
//        Q_ASSERT(serviceRigistered);

        qDebug() << QDBusConnection::systemBus().lastError();

        bool objectRegistered = QDBusConnection::systemBus().registerObject(servicePath, serviceInterface, service, QDBusConnection::ExportAllSlots|QDBusConnection::ExportAllSignals);
//        Q_ASSERT(objectRegistered);
        if (!objectRegistered) {
            //sendErrorReply(QDBusError::Failed, QDBusConnection::systemBus().lastError());
        }

        if (!serviceRigistered || !objectRegistered)
            successed = false;

        qDebug() << QDBusConnection::systemBus().lastError();
    }

    return successed;
}
